#ifndef __DEFAULT_TEAPOT_SCENE_CONTENT_H__
#define __DEFAULT_TEAPOT_SCENE_CONTENT_H__

// Includes system headers
#include <GL/glut.h>

// Includes Projection Designer headers
#include <projdesigner/include/interfaces.h>


/**
 * This class implements the contents of the teapot scene of the default plugin.
 *
 * The teapot is a basic scene content. It only has 2 parameters, which are the
 * flag that determine if the teapot will be drawn using solid or wireframe
 * representation, and the width of the lines for the wireframe representation.
 *
 */
class DefaultTeapotSceneContent: public SceneContent
{

public:


    /**
     * Default constructor
     *
     */
    DefaultTeapotSceneContent (SceneInterface* plugin);


    /**
     * Copy constructor.
     *
     * Uses copy_from() to make the copy.
     *
     */
    DefaultTeapotSceneContent (const DefaultTeapotSceneContent& data): SceneContent(data) {copy_from(data);}


    /**
     * Destructor.
     *
     */
    ~DefaultTeapotSceneContent ();


    /**
     * Implements the copy from other contents.
     *
     * This method is used by the copy constructor.
     *
     * @param content The source content to copy from.
     *
     * @return true if the copy is ok, false otherwise. The copy can fail if
     *         SceneContent is not a DefaultTeapotSceneContent.
     *
     * @seealso is_equal_to
     *
     */
    bool copy_from(const SceneContent& content);


    /**
     * Implements the egality-test with other contents.
     *
     * This could be an operator==.
     *
     * @param content The content to compare with.
     *
     * @return true if the content are equal, false otherwise. The test can
     *         fail if the contents have different parameters, but also it
     *         SceneContent is not a DefaultTeapotSceneContent.
     *
     * @seealso copy_from
     *
     */
    bool is_equal_to(const SceneContent& content) const;


    /**
     * Draws the teapot.
     *
     * @param glWidget The QGLWidget that own the OpenGL context.
     * @param cameraMatrix The camera matrix that must be use.
     *
     */
    void draw(QGLWidget* glWidget, const float* cameraMatrix);


    /**
     * Resets the teapot.
     *
     * This is a pure virtual method of SceneContent.
     * This one does nothing.
     *
     */
    void reset (void) {}


    /**
     * Gets the wireframe flag of the teapot.
     *
     * @return The wireframe flag of the teapot.
     *
     */
    bool wireframe(void) const {return m_bWireframe;}


    /**
     * Sets the wireframe flag of the teapot.
     *
     * @param wireframe The new wireframe flag of the teapot.
     *
     */
    void setWireframe(bool wireframe) {m_bWireframe=wireframe;}


    /**
     * Gets the width of the lines of the teapot.
     *
     * @return The width of the lines of the teapot.
     *
     */
    int width(void) const {return m_iWidth;}


    /**
     * Sets the width of the lines of the teapot.
     *
     * @param width The new width of the lines of the teapot.
     *
     */
    void setWidth(int width) {m_iWidth=width;}


    /**
     * Loads the teapot from a QDomElement.
     *
     * @param element The QDomElement to read from.
     *
     */
    void initFromDOMElement(const QDomElement& element);


    /**
     * Writes the teapot in a QDomDocument.
     *
     * @param name The name of the element.
     * @param doc The document whom belongs the element.
     *
     * @return The QDomElement to add.
     *
     */
    QDomElement domElement(const QString& name, QDomDocument& doc) const;


    /**
     * Gets the name of teapot scene type, stored in g_scene_name.
     *
     * @return The name of teapot scene type, stored in g_scene_name.
     *
     */
    const QString& scene_name(void) const {return g_scene_name;}


    /// Public static const scene name for the teapot scene.
    static const QString g_scene_name;


private:


    /// The wireframe flag.
    /// true means that the teapot will be draw in wireframe.
    /// false means that a solid teapot will be drawn.
    int m_bWireframe;


    /// Width of the lines of the teapot.
    /// The width is specified in pixels.
    int m_iWidth;

};


#endif // __DEFAULT_TEAPOT_SCENE_CONTENT_H__
